from django.conf import settings
from django.urls import path

from . import views


def dbfiles_url(
    prefix=settings.MEDIA_URL,
    view=views.DBFileView.as_view(),
    name="db_file",
    **kwargs,
):
    prefix = prefix.lstrip("/")
    if prefix and not prefix.endswith("/"):
        prefix += "/"
    return path("{}<path:name>".format(prefix), view, name=name, kwargs=kwargs)


urlpatterns = [
    dbfiles_url(),
]



import mimetypes
import time

import django
from django.http.response import HttpResponse, HttpResponseNotModified
from django.shortcuts import get_object_or_404
from django.utils.http import http_date
from django.views.generic.base import View
from django.views.static import was_modified_since

from .models import DBFile


class DBFileView(View):
    def get(self, request, name):
        db_file = get_object_or_404(DBFile.objects.defer("content"), name=name)

        mtime = time.mktime(db_file.updated_on.timetuple())
        kwargs = {
            "header": self.request.META.get("HTTP_IF_MODIFIED_SINCE"),
            "mtime": mtime,
        }
        if django.VERSION < (4, 1):
            # The `size` parameter was removed in Django 4.1
            kwargs["size"] = db_file.size
        modified = was_modified_since(**kwargs)

        if not modified:
            return HttpResponseNotModified()

        content_type, encoding = mimetypes.guess_type(db_file.name)
        content_type = content_type or "application/octet-stream"

        response = HttpResponse(db_file.content, content_type=content_type)
        response["Last-Modified"] = http_date(mtime)
        response["Content-Length"] = db_file.size
        if encoding:
            response["Content-Encoding"] = encoding
        return response



import importlib.metadata

__version__ = importlib.metadata.version("django-dbfiles")
__version_info__ = tuple(
    int(num) if num.isdigit() else num for num in __version__.split(".")
)

default_app_config = "dbfiles.apps.DBFilesConfig"



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-24 23:31
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="DBFile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content", models.BinaryField()),
                ("name", models.CharField(max_length=255, unique=True)),
                ("size", models.IntegerField(default=0)),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True)),
            ],
            options={"verbose_name": "DB file", "db_table": "db_file"},
        ),
    ]



from django.apps import apps
from django.core.management.base import BaseCommand
from django.db import models

from dbfiles.models import DBFile
from dbfiles.storage import DBStorage


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            "--clear",
            action="store_true",
            default=False,
            help="Deletes orphaned db_file rows, instead of just listing them",
        )

    def handle(self, *args, **options):
        # Grab a set of all known files in the database.
        db_files = set(DBFile.objects.values_list("name", flat=True))
        # Loop through every model we know about, and find any FileField (or ImageField)
        # whose storage is DBStorage. Keep a set of filenames that exist in any model in
        # the system.
        model_files = set()
        for config in apps.get_app_configs():
            for model in config.get_models():
                for field in model._meta.get_fields():
                    if isinstance(field, models.FileField) and issubclass(
                        field.storage.__class__, DBStorage
                    ):
                        model_files.update(
                            model.objects.order_by().values_list(field.name, flat=True)
                        )
        # Orphans are then files that exist in db_file but nowhere else in the system.
        orphans = db_files - model_files
        for filename in sorted(orphans):
            if options["clear"]:
                print("Deleting %s" % filename)
                DBFile.objects.filter(name=filename).delete()
            else:
                print(filename)



import datetime
import os

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils import timezone

from dbfiles.models import DBFile


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            "--clear",
            action="store_true",
            default=False,
            help="Clears out the db_file table before importing",
        )
        parser.add_argument(
            "--path",
            default=settings.MEDIA_ROOT,
            help="The root directory to import into the db_file table",
        )

    def handle(self, *args, **options):
        if options["clear"]:
            DBFile.objects.all().delete()
        media_root = os.path.abspath(options["path"])
        for root, dirs, files in os.walk(media_root):
            for f in files:
                if f.startswith("."):
                    continue
                file_path = os.path.join(root, f)
                rel_path = os.path.relpath(file_path, media_root)
                if DBFile.objects.filter(name=rel_path).exists():
                    print('"%s" already exists in the database, skipping' % rel_path)
                    continue
                mtime = os.path.getmtime(file_path)
                mod_time = timezone.make_aware(
                    datetime.datetime.utcfromtimestamp(mtime), datetime.timezone.utc
                )
                with open(file_path, "rb") as f:
                    print('Importing "%s"' % rel_path)
                    DBFile.objects.create(
                        content=f.read(),
                        name=rel_path,
                        created_on=mod_time,
                        updated_on=mod_time,
                    )



from django.db import models


class DBFile(models.Model):
    content = models.BinaryField(editable=False)
    name = models.CharField(max_length=255, unique=True)
    size = models.IntegerField(default=0)
    created_on = models.DateTimeField(auto_now_add=True)
    updated_on = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = "db_file"
        verbose_name = "DB file"

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        self.size = len(self.content)
        super(DBFile, self).save(*args, **kwargs)
